/***************************************************************************
 *   Copyright (C) 2005 by Kevin McBride          *
 *   kevin@planetsaphire.com            *
 *             *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as  *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.        *
 *             *
 *   This program is distributed in the hope that it will be useful,  *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 *   GNU General Public License for more details.      *
 *             *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the       *
 *   Free Software Foundation, Inc.,         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.   *
 ***************************************************************************/

#define	LIBTEKLTI_EXPORT	(0)

#ifndef	_cplusplus
#define	_cplusplus
#endif	/* _cplusplus */
#include "teklti.h"

#include <limits.h>
#include <errno.h>


MKTEKUUID(UTekFileASCIIBare, "7F3FE3E2-1E9E-11DA-93FA-000BDBC434D9")
MKTEKDEBUGSTRING(DBTekFileASCIIBare, "TekFileASCIIBare")


#ifndef	USE_386_ASM


/**
    Instantizer.
 */

TekFileASCIIBare::TekFileASCIIBare()
{
	TEKSTD_IMPLEMENT_INTERFACE(UTekFile)
	TEKSTD_IMPLEMENT_INTERFACE(UTekFileASCII)
	TEKSTD_IMPLEMENT_INTERFACE(UTekFileASCIIBare)
	TEKSTD_IMPLEMENT_ENDBASE

	/* NULLify all values. */
	this->PrivFileName.uchar_t_ascii = NULL;
	this->PrivFileName.uchar_t_asciilen = 0;
	this->PrivFileDesc = NULL;
	PrivFileMode = TekFileMode_ReadOnly;
	PrivTekFileType = TekFileType_ASCII;

	/* Set up our Private Buffer. */
	this->PrivBuffer.uchar_t_ascii = 
		(char *)calloc(TEKFILEASCII_STDBUFLEN + 1, sizeof(char));
	if ( this->PrivBuffer.uchar_t_ascii == NULL )
		throw;
	this->PrivBuffer.uchar_t_asciilen = 0;
	this->PrivBufferPtr.uchar_t_ascii = NULL;
	this->PrivBufferPtr.uchar_t_asciilen = 0;
}


/**
    Destructor.
 */

TekFileASCIIBare::~TekFileASCIIBare()
{
	if ( this->PrivFileDesc != NULL )
		fclose(this->PrivFileDesc);
	if ( this->PrivFileName.uchar_t_ascii != NULL )
		free(this->PrivFileName.uchar_t_ascii);
#ifdef	TEKLTI_ENFORCE_PRIVACY
	memset(this->PrivBuffer.uchar_t_ascii, '\0', TEKFILEASCII_STDBUFLEN);
#endif	/* TEKLTI_ENFORCE_PRIVACY */
	free(this->PrivBuffer.uchar_t_ascii);
}


const char TekFileASCIIBare_ReadOnly[] = "rb";
const char TekFileASCIIBare_WriteOnly[] = "wb";
const char TekFileASCIIBare_Append[] = "ab";
const char TekFileASCIIBare_ReadWrite[] = "r+b";
const char TekFileASCIIBare_ReadWriteTrunc[] = "w+b";
const char TekFileASCIIBare_ReadAppend[] = "a+b";


/**
    \fn TekFileASCIIBare::ReadLine(uchar_t** Buffer, unsigned long ReadMax)
    
    Reads up to ReadMax characters from a file and puts it in a
    uchar_t.  The Buffer can be an already-created uchar_t, or
    can be a pointer to a uchar_t* to store the new uchar_t in.
    
    NOTE: It is best to let this function create a uchar_t for
    your program/library to read from.  The uchar_t passed in
    must be at least TEKFILEASCII_STDBUFLEN + 1 bytes long.
    This is for optimization reasons, so it is best to use only
    uchar_t's that were created from this function.

  A list of \b events:
  <ul>
  <li> mouse events
  <li> keyboard events
  </ul>

  A list of \b events:  <ul>
  <li> mouse events
  <li> keyboard events
  </ul>

  A list of <b> events: </b>  <ul>
  <li> mouse events
  <li> keyboard events
  </ul>
 */

TEKERR TekFileASCIIBare::ReadLine(uchar_t** Buffer, unsigned long ReadMax)
{
	uchar_t * ubuffer;
	unsigned char wemakebuf;
	unsigned long l;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	if ( ReadMax > TEKFILEASCII_STDBUFLEN )
		ReadMax = TEKFILEASCII_STDBUFLEN;

	/* Get ready to check the passed-in pointer. */
	ubuffer = *Buffer;
	wemakebuf = 0;

	if ( ubuffer == NULL )
	{
		/* Caller wants us to create a uchar_t. */
		ubuffer = (uchar_t *)malloc(sizeof(uchar_t));
		/* If malloc() didn't fail, try to allocate the string. */
		if ( ubuffer == NULL )
		{
			return TEKERR_MEMORY;
		} else {
			ubuffer->uchar_t_ascii = (char *)
				calloc(TEKFILEASCII_STDBUFLEN + 1, sizeof(char));
			if ( ubuffer->uchar_t_ascii == NULL )
			{
				/* If calloc() failed, there's nothing to do. */
				free(ubuffer);
				return TEKERR_MEMORY;
			}

			/* Set up the length value. */
			ubuffer->uchar_t_asciilen =
				(TEKFILEASCII_STDBUFLEN + 1) * sizeof(char);

			/* Set 'wemakebuf' to 1. */
			wemakebuf = 1;
		}
	}
#ifdef	TEKLTI_ENFORCE_PRIVACY
	else
	{
		/* Clear out the memory. */
		memset(
			ubuffer->uchar_t_ascii,
			'\0',
			(TEKFILEASCII_STDBUFLEN + 1) * sizeof(char)
		);
	}
#endif	/* TEKLTI_ENFORCE_PRIVACY */

	/* If we haven't yet cleared out our buffer... */
	if ( this->PrivBuffer.uchar_t_asciilen > 0 )
	{
		/* Copy some characters by hand. */
		for ( l=0; l < ReadMax; l++ )
		{
			/* Check to see if we have already reached EOS. */
			if ( this->PrivBuffer.uchar_t_asciilen == 0 )
			{
				ubuffer->uchar_t_ascii[l] = '\0';
				/* Restore the PrivBuffer pointer. */
				this->PrivBuffer.uchar_t_ascii =
					this->PrivBufferPtr.uchar_t_ascii;
				break;
			}

			if ( this->PrivBuffer.uchar_t_ascii[0] == '\n' )
			{
ReadFileEOS:
				ubuffer->uchar_t_ascii[l] = '\0';
				/* Set asciilen to zero. */
				this->PrivBuffer.uchar_t_asciilen = 0;
				/* Restore the PrivBuffer pointer. */
				this->PrivBuffer.uchar_t_ascii =
					this->PrivBufferPtr.uchar_t_ascii;
				return TEKERR_FILE_EOF;
			}
			if ( this->PrivBuffer.uchar_t_ascii[0] == '\r' )
				goto ReadFileEOS;

			/* Copy a character. */
			ubuffer->uchar_t_ascii[l] = this->PrivBuffer.uchar_t_ascii[0];
			/* Decrement the length. */
			this->PrivBuffer.uchar_t_asciilen--;
			/* Increment the pointer. */
			this->PrivBuffer.uchar_t_ascii++;
		}

		ubuffer->uchar_t_ascii[ReadMax] = '\0';

		/* Check to see if we have already reached EOS. */
		if ( this->PrivBuffer.uchar_t_asciilen == 0 )
		{
			/* Restore the PrivBuffer pointer. */
			this->PrivBuffer.uchar_t_ascii =
				this->PrivBufferPtr.uchar_t_ascii;
		}

		/* We're done here. */
		return TEKERR_OK;
	}

	/* Read the file. */
	if ( 
		fgets(
			this->PrivBuffer.uchar_t_ascii,
			TEKFILEASCII_STDBUFLEN + 1,
			this->PrivFileDesc
		) == NULL
	)
	{
		/* An error occured; cleanup. */
		if ( wemakebuf )
		{
			free(ubuffer->uchar_t_ascii);
			free(ubuffer);
		}

		/* Check for end of file status. */
		if ( feof(this->PrivFileDesc) )
			return TEKERR_FILE_EOF;

		/* Check for EPIPE error. */
		if ( errno == EPIPE )
			return TEKERR_FILE_PIPE;

		/* All other errors are considered IO errors. */
		return TEKERR_FILE_IO;
	}

	/* Remember the origional pointer. */
	this->PrivBufferPtr.uchar_t_ascii = this->PrivBuffer.uchar_t_ascii;

	/* Get the len. */
	l = (unsigned long)strlen(this->PrivBuffer.uchar_t_ascii);
	/* Test to see if ReadMax is greater than the bytes we read. */
	if ( ReadMax >= l )
	{
		/* Set PrivBuffer len to zero. */
		this->PrivBuffer.uchar_t_asciilen = 0;
		/* Copy our string. */
		strcpy(ubuffer->uchar_t_ascii, this->PrivBuffer.uchar_t_ascii);
		if ( wemakebuf )
			*Buffer = ubuffer;
		return TEKERR_OK;
	}

	/* Store the len()-ReadMax. */
	this->PrivBuffer.uchar_t_asciilen = l - ReadMax;

	/* Copy the string. */
	strncpy(
		ubuffer->uchar_t_ascii,
		this->PrivBuffer.uchar_t_ascii, ReadMax
	);
	ubuffer->uchar_t_ascii[ReadMax] = '\0';
	this->PrivBuffer.uchar_t_ascii += ReadMax;

	/* Set the new pointer into Buffer as needed. */
	if ( wemakebuf )
		*Buffer = ubuffer;

	/* We're done here. */
	return TEKERR_OK;
}

/**
    \fn TekFileASCIIBare::WriteLine(const uchar_t* Buffer)
    
    Writes a ascii-part of a uchar_t to the file.

    WARNING!  On Intel platforms, this function may utilize MMX
    for optimization purposes!  Register mm6 can be
    modified by this function!
 */

TEKERR TekFileASCIIBare::WriteLine(const uchar_t* Buffer)
{
	int err;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Write out the string. */
	err = fputs(Buffer->uchar_t_ascii, this->PrivFileDesc);
	if ( err == EOF )
	{
WriteLineEOF:
		/* We have to find out what the error is. */
		switch ( errno )
		{
			case EPIPE:
				return TEKERR_FILE_PIPE;
			case ENOSPC:
				return TEKERR_FILE_DISKFULL;
			case EBADF:
				/* Check for read-only status. */
				if ( this->PrivFileMode == TekFileMode_ReadOnly )
					return TEKERR_FILE_READONLY;
			case EIO:
			default:
				return TEKERR_FILE_IO;
		}
	}

	/* Terminate the line. */
#ifdef	TEKLTI_WRITELINE_CRLF
	err = fputs("\r\n", this->PrivFileDesc);
#else	/* not TEKLTI_WRITELINE_CRLF */
	err = fputc('\n', this->PrivFileDesc);
#endif	/* not TEKLTI_WRITELINE_CRLF */
	/* Check for an error. */
	if ( err == EOF )
		goto WriteLineEOF;

	return TEKERR_OK;
}

/**
    \fn TekFileASCIIBare::get_FILE(FILE ** FileDescriptor)
    
    Obtains the handle used to access the open file.
 */
TEKERR TekFileASCIIBare::get_FILE(FILE ** FileDescriptor)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( FileDescriptor == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	*FileDescriptor = this->PrivFileDesc;
	
	return TEKERR_OK;
}


/**
    \fn TekFileASCIIBare::get_Filename(uchar_t ** Name)
    
    Obtains the filename in the form of a uchar.
 */
TEKERR TekFileASCIIBare::get_Filename(uchar_t ** Name)
{
	uchar_t * newval;
	
#ifndef	NO_SIZE_T_CHECKS
	if ( Name == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileName.uchar_t_ascii == NULL )
		return TEKERR_UNEXPECTED;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Convert our ASCII string to a uchar. */
	newval = char2uchar(this->PrivFileName.uchar_t_ascii);
	if ( newval == NULL )
		return TEKERR_MEMORY;

	/* Set up our uchar. */
	*Name = newval;

	/* Return to caller. */
	return TEKERR_OK;
}


/**
    \fn TekFileASCIIBare::put_FILE(FILE * FileDescriptor)
    
    Sets the handle used to access a file.\n
    \n
    WARNING!!!!!  DO NOT USE THE FILE DESCRIPTOR YOU PASS IN \n
    TO THIS FUNCTION UNLESS YOU HAVE RE-RETRIEVED IT USING \n
    TekFileASCIIBare::get_FILE()!  Utilizing the descriptor in a direct fashion after a \n
    successful call to put_FILE() will result in undefined results!!!!!
 */
TEKERR TekFileASCIIBare::put_FILE(FILE * FileDescriptor)
{
	this->PrivFileDesc = FileDescriptor;

	return TEKERR_OK;
}


/**
    \fn TekFileASCIIBare::put_Filename(uchar_t * Name)
    
    Sets the filename to access.
 */
TEKERR TekFileASCIIBare::put_Filename(uchar_t * Name)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( Name == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	this->PrivFileName.uchar_t_ascii = strdup(uchar2char(Name));

	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::get_Type(TekFileType * FileTypePointer)
    
    Gets the file type.
 */
TEKERR TekFileASCIIBare::get_Type(TekFileType * FileTypePointer)
{
	TekFileType * retval;

	retval = FileTypePointer;

#ifndef	NO_SIZE_T_CHECKS
	if ( FileTypePointer == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	retval[0] = this->PrivTekFileType;

	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::Close()
    
    Closes the file.
 */
TEKERR TekFileASCIIBare::Close()
{
	int err;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	err = fclose(this->PrivFileDesc);

	if ( err != 0 )
	{
		switch ( err )
		{
			case ENOSPC:
				return TEKERR_FILE_DISKFULL;
			case EIO:
			case EPIPE:
				return TEKERR_FILE_IO;
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	this->PrivFileDesc = NULL;

	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::get_Mode(TekFileMode * ModeID)
    
    Retrieves the mode currently set.
 */
TEKERR TekFileASCIIBare::get_Mode(TekFileMode * ModeID)
{
#ifndef	NO_SIZE_T_CHECKS
	if ( ModeID == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

	*ModeID = this->PrivFileMode;

	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::Flush()
    
    Flushes the buffer.
 */
TEKERR TekFileASCIIBare::Flush()
{
	int err;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	err = fflush(this->PrivFileDesc);

	if ( err != 0 )
	{
		switch ( err )
		{
			case ENOSPC:
				return TEKERR_FILE_DISKFULL;
			case EIO:
			case EPIPE:
				return TEKERR_FILE_IO;
			default:
				return TEKERR_UNEXPECTED;
		}
	}

	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::Open()
    
    Open the file.
 */
TEKERR TekFileASCIIBare::Open()
{
	const char * tmpopenvar;

	/* Make sure this instance does not have a file already open. */
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Figure out what fopen() op are going to do. */
	switch ( this->PrivFileMode )
	{
		case TekFileMode_WriteOnly:
			tmpopenvar = TekFileASCIIBare_WriteOnly;
			break;
		case TekFileMode_Append:
			tmpopenvar = TekFileASCIIBare_Append;
			break;
		case TekFileMode_ReadWrite:
			tmpopenvar = TekFileASCIIBare_ReadWrite;
			break;
		case TekFileMode_ReadWriteTrunc:
			tmpopenvar = TekFileASCIIBare_ReadWriteTrunc;
			break;
		case TekFileMode_ReadAppend:
			tmpopenvar = TekFileASCIIBare_ReadAppend;
			break;
		default:
			tmpopenvar = TekFileASCIIBare_ReadOnly;
			break;
	}

	/* Open the file. */
	this->PrivFileDesc = fopen(
		this->PrivFileName.uchar_t_ascii,
		tmpopenvar
	);
	if ( this->PrivFileDesc == NULL )
		return TEKERR_FAIL;

	/* Return to caller. */
	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::Open64()
    
    Open the file using 64 bit semantics.
 */
TEKERR TekFileASCIIBare::Open64()
{
#ifndef	OPEN64_NOT_SUPPORTED
	const char * tmpopenvar;

	/* Make sure this instance does not have a file already open. */
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Figure out what fopen() op are going to do. */
	switch ( this->PrivFileMode )
	{
		case TekFileMode_ReadOnly:
			tmpopenvar = TekFileASCIIBare_ReadOnly;
			break;
		case TekFileMode_WriteOnly:
			tmpopenvar = TekFileASCIIBare_WriteOnly;
			break;
		case TekFileMode_Append:
			tmpopenvar = TekFileASCIIBare_Append;
			break;
		case TekFileMode_ReadWrite:
			tmpopenvar = TekFileASCIIBare_ReadWrite;
			break;
		case TekFileMode_ReadWriteTrunc:
			tmpopenvar = TekFileASCIIBare_ReadWriteTrunc;
			break;
		case TekFileMode_ReadAppend:
			tmpopenvar = TekFileASCIIBare_ReadAppend;
			break;
		default:
			return TEKERR_UNAVAILABLE;
	}

	/* Open the file. */
	this->PrivFileDesc = fopen64(
		this->PrivFileName.uchar_t_ascii,
		tmpopenvar
	);
	if ( this->PrivFileDesc == NULL )
		return TEKERR_FAIL;

	/* Return to caller. */
	return TEKERR_OK;
#else	/* OPEN64_NOT_SUPPORTED */
	return TEKERR_NOTIMPL;
#endif	/* OPEN64_NOT_SUPPORTED */
}


/*!
    \fn TekFileASCIIBare::put_Mode(TekFileMode ModeID)
    
    Sets the mode used to open the file.
 */
TEKERR TekFileASCIIBare::put_Mode(TekFileMode ModeID)
{
#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc != NULL )
		return TEKERR_ACCESS;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	this->PrivFileMode = ModeID;
	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
    
    Read from the file using 64 bit semantics.
 */
TEKERR TekFileASCIIBare::Read64(void * Buffer, uint64_t BytesToRead, uint64_t * BytesRead)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileASCIIBare::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
    
    Read from the file.
 */
TEKERR TekFileASCIIBare::Read(void * Buffer, unsigned long BytesToRead, unsigned long * BytesRead)
{
	size_t bytesread;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Write the buffer to the file. */
	bytesread = fread(Buffer, sizeof(char), BytesToRead, this->PrivFileDesc);

	/* Put the result into BytesWrote. */
	*BytesRead = bytesread;

	/* Check for success or failure. */
	if ( bytesread > 0 )
		return TEKERR_OK;

	/* Check for EOF error. */
	if ( feof(this->PrivFileDesc) != 0 )
		return TEKERR_FILE_EOF;

	/* Check for EPIPE error. */
	if ( errno == EPIPE )
		return TEKERR_FILE_PIPE;

	/* All other errors are considered IO errors. */
	return TEKERR_FILE_IO;
}


/*!
    \fn TekFileASCIIBare::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
    
    Seek to another position of the file using 64 bit semantics.
 */
TEKERR TekFileASCIIBare::Seek64(TekFileSeek SeekWhere, int64_t NewLocation)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileASCIIBare::Seek(TekFileSeek SeekWhere, signed long NewLocation)
    
    Seek to another position of the file.
 */
TEKERR TekFileASCIIBare::Seek(TekFileSeek SeekWhere, signed long NewLocation)
{
	int SeekWhence;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
  		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	switch ( SeekWhere )
	{
		case TekFileSeek_Begin:
			SeekWhence = SEEK_SET;
			break;
		case TekFileSeek_Current:
			SeekWhence = SEEK_CUR;
			break;
		case TekFileSeek_End:
			SeekWhence = SEEK_END;
			break;
		default:
			return TEKERR_UNAVAILABLE;
	}

	if ( fseek(this->PrivFileDesc, NewLocation, SeekWhence) )
		return TEKERR_FAIL;

	return TEKERR_OK;
}


/*!
    \fn TekFileASCIIBare::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
    
    Write to the file using 64 bit semantics.
 */
TEKERR TekFileASCIIBare::Write64(void * Buffer, uint64_t BytesToWrite, uint64_t * BytesWrote)
{
	return TEKERR_NOTIMPL;
}


/*!
    \fn TekFileASCIIBare::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
    
    Write to the file.
 */
TEKERR TekFileASCIIBare::Write(void * Buffer, unsigned long BytesToWrite, unsigned long * BytesWrote)
{
	size_t written;

#ifndef	NO_SIZE_T_CHECKS
	if ( Buffer == NULL )
		return TEKERR_POINTER;
#endif	/* not NO_SIZE_T_CHECKS */

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Write the buffer to the file. */
	written = fwrite(Buffer, sizeof(char), BytesToWrite, this->PrivFileDesc);

	/* Put the result into BytesWrote. */
	*BytesWrote = written;

	/* Check for success or failure. */
	if ( written == BytesToWrite )
		return TEKERR_OK;

	/* We have to find out what the error is. */
	switch ( errno )
	{
		case EPIPE:
			return TEKERR_FILE_PIPE;
		case ENOSPC:
			return TEKERR_FILE_DISKFULL;
		case EBADF:
			/* Check for read-only status. */
			if ( this->PrivFileMode == TekFileMode_ReadOnly )
				return TEKERR_FILE_READONLY;
		case EIO:
		default:
			return TEKERR_FILE_IO;
	}
}


/*!
    \fn TekFileASCIIBare::get_Location(unsigned long * Location)

    Retrieves the location.
 */

TEKERR TekFileASCIIBare::get_Location(unsigned long * Location)
{
	long loc;

#ifndef	NO_INTERNAL_NULL_CHECKS
	if ( this->PrivFileDesc == NULL )
		return TEKERR_SEQUENCE;
#endif	/* not NO_INTERNAL_NULL_CHECKS */

	/* Get the current file position. */
	loc = ftell(this->PrivFileDesc);

	/* Check for error. */
	if ( loc != -1 )
	{
		/* Move location to memory referenced by Location. */
		*Location = (unsigned long)loc;

		/* Indicate success. */
		return TEKERR_OK;
	}

	/* See if this is a pipe error. */
	if ( errno != EPIPE )
		return TEKERR_FILE_IO;

	return TEKERR_FILE_PIPE;
}


/*!
    \fn TekFileASCIIBare::get_Location64(uint64_t * Location)

    Retrieves the location and stores it in a 64-bit variable.
 */

TEKERR TekFileASCIIBare::get_Location64(uint64_t * Location)
{
	return TEKERR_NOTIMPL;
}


#endif	/* not USE_386_ASM */


